clear all
close all


% ALTERNATING MINIMIZATION PARAMETERS
% maximal number of iterations for alternating minimization
T = 50;
% convergence accuracy of alternating minimization
epsilon = 0.01;

% DATA PARAMETERS
% number of observations
m = 500;
% dimension of regression model: our solvers are only for n=3,4. For n>4
% please use a general polynomial system solver such as Bertini.
n = 4;
% standard deviation of additive noise
sigma = 0.01;


% generate data
xi_star = randn(n,1);
A = randn(m,n);
y = A * xi_star + sigma*randn(m,1);

% obtain the roots of the polynomial system of equations
if n==3
    Xi_AI = unique(sympol3Mat(A, y), 'rows').';
elseif n==4
    Xi_AI = unique(sympol4Mat(A, y), 'rows').';
else
    display("error: for n>4 please use Bertini or some other polynomial system solver");
end
    

% find the error of the best root of the polynomial system 
E_AI = vecnorm(Xi_AI-xi_star, 2, 1)/norm(xi_star);
[e_AI_star, ~] = min(E_AI);

% estimate the best root of the polynomial system
E_AI_hat = vecnorm(sort(y) - sort(A*Xi_AI, 1), 2, 1);
[~, index] = min(E_AI_hat);
xi_AI_hat = squeeze(Xi_AI(:, index));
e_AI_hat = norm(xi_AI_hat-xi_star)/norm(xi_star);

% run alternating minimization to compute MLE estimator
xi_AIEM = SLR_hardEM(A,y,xi_AI_hat,T,epsilon);
e_AIEM = norm(xi_AIEM-xi_star)/norm(xi_star);

% errors
display('xi_AI_star, xi_AI_hat, xi_AIEM');
[e_AI_star, e_AI_hat, e_AIEM]



