function [Xreturn,X,mycost] = mimoblinddemixing(Ksize,X0,Functrue,Ax,y,options,epsilon)

if ~isfield(options,'maxiter'); options.maxiter = 100; end
if ~isfield(options,'verbosity'); options.verbosity = 20; end
if ~isfield(options,'ranktol'); options.ranktol = 1e-6; end

maxiter = options.maxiter;
verb = 1;
ranktol = options.ranktol;

Ksize = num2cell(Ksize,1);
[n1,n2,S,R,m,~] = deal(Ksize{:});
truevalue = Functrue.trueValue;
truetheta = Functrue.trueTheta;

r = 1;
[u0,s0,v0] = cellfun(@(x)svd(x),X0,'Uni',0);

fid1 = fopen('test/DC_print1.txt','a+');
fid2 = fopen('test/DC_print2.txt','a+');
[lastobj,lastcost] = deal(0);
s00 = cellfun(@(x)zeros(n1,n2*R),s0,'Uni',0);
for ii = 1:S
    s00{ii}(1,1) = 1;
end
    
for iter = 1:maxiter
    
   
    partial_X =  cellfun(@(x,y,z)x*y*z',u0,s00,v0,'Uni',0);
    
    
    %% optimization
    cvx_begin quiet
    variable X(n1, n2*R, S) complex
    
    % objective function
    expression obj
    obj = 0;
    for iS =1:S
       tmp1 = X(:,:,iS);
        obj = obj + norm_nuc(tmp1)-real(partial_X{iS,1}(:)'*tmp1(:));
    end
    
    
    minimize obj
    
    % constraints
    expression y_p(m, R)
    for jR = 1:R
         tmp2 = X(:,(jR-1)*n2+1:jR*n2,:);
         y_p(:,jR) = Ax{jR}*tmp2(:);
    
    end
    
    subject to
      norm(y_p-y,'fro')<= epsilon
         
    cvx_end
    
    X = reshape(arrayfun(@(len)X(:,:,len),1:S,'Uni',0),[S,1]);
    
    [u0,s0,v0] = cellfun(@(x)svd(x),X,'Uni',0);
    
    s0 = cellfun(@(x)diag(x),s0,'Uni',0);
    mycost  = sum(cell2mat(cellfun(@(x)sum(x(r+1:end)),s0,'Uni',0)));
    err = abs(lastobj - cvx_optval)/max(1,cvx_optval);
    err1 = abs(lastcost - mycost)/max(1,mycost);
    err2 =  sqrt(sum(cellfun(@(x,y)norm(x-y,'fro')^2,X,truetheta)))/truevalue;
    
    if verb && rem(iter,4) == 0
        fprintf(fid1,'obj:%.3e',cvx_optval);
        fprintf(fid1,' iter:%.3d,err:%.3e,err1:%.3e,err2:%.3e,mycost:%.3e\n',iter,err,err1,err2,mycost);
    end
    if mycost < 1e-6
        fprintf(fid2,'obj:%.3e',cvx_optval);
        fprintf(fid2,' iter:%.3d,err:%.3e,err1:%.3e,err2:%.3e,mycost:%.3e\n',iter,err,err1,err2,mycost);
        break;
    end
    lastobj = cvx_optval;
    lastcost = mycost;
end

fclose(fid1);
fclose(fid2);
 Xreturn=cell(S,1);  
for idx = 1:S
    [u,iS,v] = svd(X{idx});
    rx = 1;
    %rx = rank(X{idx},ranktol );
    iS(rx+1:end,rx+1:end)=0;
    Xreturn{idx}=u*iS*v'; 
end
    
end


