% the ManOpt tool needs to be installed via running manopt/importmanopt.m
clear all;
clc
rng('default');rng(1);warning off;
addpath('./fun');

testnum =20;
size_set =[600];
size_len = length(size_set);
s = 20;
lambda =0.08;

sparse=15;
n1 = 10; n2 =10;

r=1;
params.verbosity =0;
params.costtol = 1e-6;
params.maxiter = 500;
params.tolgradnorm = 1e-6;
params.eps = 1e-3;
params.lambda = lambda;
snr_set_index =[0.5:1];
sigma_set = 10.^(snr_set_index);
size_snr = size(sigma_set,2);

allcost0 = zeros(size_len,size_snr,testnum);
allcost1 = zeros(size_len,size_snr,testnum);
allcost2 = zeros(size_len,size_snr,testnum);
allcost3 = zeros(size_len,size_snr,testnum);
allcost4 = zeros(size_len,size_snr,testnum);

for t=1:size_len
    m = size_set(t);
    for ks = 1:size_snr
        Ksize = [n1,n2,s,m];
        snr = sigma_set(ks);
parfor ki = 1:testnum
        [X,y,C,x,h,g,z,y1 ] = generate_model( m,n1,n2,s,sparse);
        omega = randn(m,1)+1i*randn(m,1);
        y = y+snr*norm(y)*(omega/norm(omega))/sqrt(2);
        [A,B,FC] = generate_decon_matrix(m,n1,n2,s,C);
%%  RTR   
        [ h0,x0,d,mu] = RGD_initial(Ksize,A,B,FC,y); 
        X0_re =cellfun(@(h,x)[h;x],h0,x0,'Uni',0);

        [Xout0, infos1]=Riemannian_fixedrank_TR(r, Ksize,X0_re, params, A, y);
        x1 = cellfun(@(x,y)x(1:n1,:)*x(n1+1:end,:)',Xout0,'Uni',0);
        allcost0(t,ks,ki)=sqrt(sum(cellfun(@(x,y)norm(x(1:n1,:)*x(n1+1:end,:)'-y,'fro')^2,Xout0,X))/sum(cellfun(@(x,y)norm(x,'fro')^2,X)));
        fprintf('SNR:%d,RTR: m:%.3d, K:%.2d, test:%.3d, cost:%.4e\n',ks,m,s, ki,allcost0(t,ks,ki));
  
%% nuclear
        Xout1 = convex_cvx1( Ksize, A, y,1,0,0);
        allcost1(t,ks,ki)=sqrt(sum(cellfun(@(x,y)norm(x-y,'fro')^2,Xout1,X))/sqrt(sum(cellfun(@(x,y)norm(x,'fro')^2,X))));
        fprintf('SNR:%d, CVX: m:%.3d, K:%.2d, test:%.3d, cost:%.4e\n',ks,m,s, ki,allcost1(t,ks,ki));
%% frobius
        Xout2 = convex_cvx1( Ksize, A, y,0,1,0);
        allcost2(t,ks,ki)=sqrt(sum(cellfun(@(x,y)norm(x-y,'fro')^2,Xout2,X))/sqrt(sum(cellfun(@(x,y)norm(x,'fro')^2,X))));
        fprintf('SNR:%d, FRO:m:%.3d, K:%.2d, test:%.3d, cost:%.4e\n',m,s, ki,allcost2(t,ks,ki));
%% combine
        Xout3 = convex_cvx1( Ksize, A, y,0.4,0.5,0);
        allcost3(t,ks,ki)=sqrt(sum(cellfun(@(x,y)norm(x-y,'fro')^2,Xout3,X))/sqrt(sum(cellfun(@(x,y)norm(x,'fro')^2,X))));
        fprintf('SNR:%d,COMm:%.3d, K:%.2d, test:%.3d, cost:%.4e\n',ks,m,s, ki,allcost3(t,ks,ki));



     end
     end
end

tol = 1e-2;
test_succ0 = mean(allcost0,3);
test_succ1 = mean(allcost1,3);
test_succ2 = mean(allcost2,3);
test_succ3 = mean(allcost3,3);

figure,plot(-mag2db(sigma_set),mag2db(test_succ0(1,:)),'-s','LineWidth',1.5,'MarkerSize',6),hold on
plot(-mag2db(sigma_set),mag2db(test_succ1(1,:)),'-s','LineWidth',1.5,'MarkerSize',6),hold on
plot(-mag2db(sigma_set),mag2db(test_succ2(1,:)),'-s','LineWidth',1.5,'MarkerSize',6),hold on
plot(-mag2db(sigma_set),mag2db(test_succ3(1,:)),'-s','LineWidth',1.5,'MarkerSize',6),hold on
legend('tr','nuclear','fro','combine');

